{% extends "admin_tools/dashboard/module.html" %}
{% load admin_chart_tags %}
{% load nvd3_tags %}

{% block module_content %}
    {% if module.require_chart_jscss %}
       {% include '../../include_nvd3.html' %}
    {% endif %}

    {% if module.form_field %}
       {% get_control_form module.dashboard_stats request.user as form %}
       {% include "../chart_form.html" with chart=module.dashboard_stats %}
    {% endif %}

    {% include_container module.chart_container %}

{% endblock %}

