# Generated by Django 3.1.6 on 2021-02-21 11:47

import multiselectfield.db.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('admin_tools_stats', '0012_auto_20210207_0859'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dashboardstats',
            name='allowed_time_scales',
            field=multiselectfield.db.fields.MultiSelectField(choices=[('hours', 'Hours'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months'), ('quarters', 'Quarters'), ('years', 'Years')], default=('hours', 'days', 'weeks', 'months', 'quarters', 'years'), max_length=1000, verbose_name='Allowed time scales'),
        ),
        migrations.AlterField(
            model_name='dashboardstats',
            name='default_time_scale',
            field=models.CharField(choices=[('hours', 'Hours'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months'), ('quarters', 'Quarters'), ('years', 'Years')], default='days', max_length=90, verbose_name='Default time scale'),
        ),
    ]
